/*
 * ultra.c
 *
 * Created: 2016-06-02 10:28:34
 *  Author: dsosnowski
 */ 



#define F_CPU 16000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>

#include "hard.h"
#include "ultra.h"


#define TIMER40KHZ_TOPVAL		399
#define TIMER40KHZ_HALFVAL		199


struct {
	volatile uint16_t counter;
	volatile char run;
	uint16_t suppress;
	uint16_t pulses;
	uint16_t range;
	
} ultra;




//---------------------------------------------------
void UltraInit(void){
	TRANS_DISABLE;
	TRANS_DDR |= (1<<TRANS_ENABNE_PIN);	
	PULSE_DDR |= (1<<PULSE_NPIN | 1<<PULSE_PPIN);
}

//---------------------------------------------------
void UltraStart(char pulses, char range){
	TCCR1B = (1<<WGM13 | 1<<WGM12);
	ultra.pulses = pulses + 2;
	ultra.counter = 0;
	ultra.suppress = (ultra.pulses + DEATH_TIME);
	ultra.range = range + ultra.suppress;
	ultra.run = 1;
	
	TRANS_ENABLE;
	TCNT1 = 0;
	ICR1 = TIMER40KHZ_TOPVAL;
	OCR1A = TIMER40KHZ_HALFVAL;
	OCR1B = TIMER40KHZ_HALFVAL;

	TIFR1 |= (1<<TOV1 | 1<<OCF1A);
	TIMSK1 |= (1<<TOIE1);

	PULSE_PORT &= ~(1<<PULSE_NPIN);
	PULSE_PORT |=  (1<<PULSE_PPIN);

	TCCR1A = (1<<WGM11 | 1<<COM1B1 | 0<<COM1B0 | 1<<COM1A1 | 1<<COM1A0);
	TCCR1B = (1<<WGM13 | 1<<WGM12 | 1<<CS10);
	
	PULSE_PORT &= ~(1<<PULSE_NPIN);
	PULSE_PORT &= ~(1<<PULSE_PPIN);
}

//---------------------------------------------------
void UltraStop(void){
	TCCR1B = (1<<WGM13 | 1<<WGM12);
	TCCR1A = (1<<WGM11 | 0<<COM1B1 | 0<<COM1B0 | 0<<COM1A1 | 0<<COM1A0);
	PULSE_PORT &= ~(1<<PULSE_NPIN);
	PULSE_PORT &= ~(1<<PULSE_PPIN);
	TRANS_DISABLE;
}

//---------------------------------------------------
ISR (TIM1_OVF_vect) {
	ultra.counter++;
	if (ultra.counter == ultra.pulses) {
		TIFR1 |= (1<<OCF1A);
		TIMSK1 |= (1<<OCIE1A);
	}
	else if (ultra.counter == (ultra.pulses + 1)) {
		TCCR1A = (1<<WGM11 | 0<<COM1B1 | 0<<COM1B0 | 0<<COM1A1 | 0<<COM1A0);
	}
	else if (ultra.counter == ultra.suppress) {
		//TCCR1B = (1<<WGM13 | 1<<WGM12);
		TCCR1A = (1<<WGM11 | 0<<COM1B1 | 0<<COM1B0 | 0<<COM1A1 | 0<<COM1A0);
		TRANS_DISABLE;
		ultra.run = 2;
	}
	else if (ultra.counter >= ultra.range) {
		TCCR1B = (1<<WGM13 | 1<<WGM12);
		TCCR1A = (1<<WGM11 | 0<<COM1B1 | 0<<COM1B0 | 0<<COM1A1 | 0<<COM1A0);
		ultra.run = 0;		
	}
}

//---------------------------------------------------
ISR (TIM1_COMPA_vect) {
	TCCR1A = (1<<WGM11 | 0<<COM1B1 | 0<<COM1B0 | 1<<COM1A1 | 1<<COM1A0);
	TIMSK1 &= ~(1<<OCIE1A);
}

//---------------------------------------------------
char UltraRunGet(void){
	return ultra.run;
}