/*
 * ultra_detector.c
 *
 * Created: 2016-06-02 10:10:46
 *  Author: dsosnowski
 */ 


#define F_CPU 16000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>

#include "hard.h"
#include "ultra.h"
#include "detect.h"
#include "wdog.h"


char Analize(char detect);


struct {
	char output;
	char result;
	char tick;
	uint16_t time_on;
	uint16_t led;	
} system;

//---------------------------------------------------
int main(void)
{
	uint16_t new;
	uint16_t prev;
	char special = LED_SPECIAL;
	
	WdtSet(7);
	
	OUT_OFF;
	OUT_DDR |= (1<<OUT_PIN | 1<<LED_PIN);	
	
	UltraInit();
	DetectInit();	
	sei();
	
    //---------------------------------------------------
	while(1)
    {
		DetectTime();
		DetectRange();
		
	    //---------------------------------------------------
		UltraStart(SEND_PULSES, DetectRangeGet());
		
		DetectWaveScanInit();
		while (UltraRunGet() == 1);	
		system.result = 0;
		prev = DetectWaveScanGet();	
		while (UltraRunGet() == 2) {
			new = DetectWaveScanGet();
			//if (new < 512) {
				if (new > (prev + DETECT_SENSIVITY)) {
					system.result = 1;
				}
			//}
			prev = new;
		}	
		DetectWaveScanStop();

		if (Analize(system.result) > 0) {
			system.output = 1;
			system.tick = 0;
			system.time_on = 0;
			special = LED_SPECIAL;			
		}
		
	    //---------------------------------------------------	    
		if (system.output > 0) {
			system.tick++;
			if (system.tick >= TICKPERSEC) {
				if (system.time_on < DetectTimeGet()) {
					system.time_on++;
				} else {
					system.output = 0;
					system.time_on = 0;
				}
				system.tick = 0;
			}
		}
		
		if (system.output) {
			OUT_SET;
		} else {
			OUT_OFF;
		}
		
	    //---------------------------------------------------
		if (special > 0) {
			special--;
			system.led = 0;
		}
		if (system.led == 0) LED_SET;
		if (system.led == LED_BLINK) LED_OFF;
		system.led++;
		if (system.led >= LED_PERIOD) system.led = 0;
		
	    //---------------------------------------------------
		_delay_ms(MAIN_DELAY);
		WdtReset();
    }
}

//---------------------------------------------------
char Analize(char detect)
{
	static char detect_delay;
	static char pass;
	char result = 0;
	
		if (detect > 0) {
			if (detect_delay < (1000/MAIN_DELAY)) {
				pass++;
				detect_delay = 0;
			} else {
				pass = 0;
				detect_delay = 0;
			}
		}
		if (pass >= 3) {
			result = 1;
			pass = 0;
		}

	if (detect_delay < 0xFF) detect_delay++;
	return result;	
}